#define PatchLevel "31"

/*
 *	the Patch Level above is to identify the version
 *	of the all the files in this directory. given the above
 *	number i can pull up the correct version of any individual
 *	file in this directory. in all communications i will appreciate
 *	very much if you can quote this number.
 *
 *	$Author: bammi $
 */
