__EXTERN int  vq_tabstatus __PROTO((int handle));
__EXTERN void v_hardcopy __PROTO((int handle));
__EXTERN void v_dspcur  __PROTO((int handle, int x, int y));
__EXTERN void v_rmcur __PROTO((int handle));
__EXTERN void v_form_adv __PROTO((int handle));
__EXTERN void v_output_window __PROTO((int handle, int *pxyarray));
__EXTERN void v_clear_disp_list __PROTO((int handle));
__EXTERN void v_bit_image __PROTO((int handle, const char *filename, int aspect, int x_scale,
			int y_scale, int h_align, int v_align,
			int *pxyarray));
__EXTERN void vq_scan __PROTO((int handle, int *g_slice, int *g_page, int *a_slice,
		int *a_page, int *div_fac));
__EXTERN void v_alpha_text __PROTO((int handle, const char *string));
__EXTERN int  vs_palette __PROTO(( int handle, int palette));
__EXTERN void v_sound __PROTO((int handle, int frequency, int duration));
__EXTERN int  vs_mute __PROTO((int handle, int action));
__EXTERN void vt_resolution __PROTO(( int handle, int xres, int yres, int *xset, int *yset));
__EXTERN void vt_axis __PROTO((int handle, int xres, int yres, int *xset, int *yset));
__EXTERN void vt_origin __PROTO((int handle, int xorigin, int yorigin));
__EXTERN void vq_dimensions __PROTO((int handle, int *xdimension, int *ydimension));
__EXTERN void vt_alignment __PROTO((int handle, int dx, int dy));
__EXTERN void vsp_film __PROTO((int handle, int index, int lightness));
__EXTERN int  vqp_filmname __PROTO((int handle, int index, char *name));
__EXTERN void vsc_expose __PROTO((int handle, int state));
__EXTERN void v_meta_extents __PROTO((int handle, int min_x, int min_y, int max_x, int max_y));
__EXTERN void v_write_meta __PROTO((int handle, int num_intin, int *a_intin, int num_ptsin,
			int *a_ptsin));
__EXTERN void vm_pagesize __PROTO((int handle, int pgwidth, int pgheight));
__EXTERN void vm_coords __PROTO((int handle, int llx, int lly, int urx, int ury));
__EXTERN void vm_filename __PROTO((int handle, const char *filename));
__EXTERN void v_escape2000 __PROTO((int handle, int times));
