#ifdef __DEF_ALL__
#define L_vq_gdos
#define L_vq_vgdos
#endif /* __DEF_ALL__ */

	TEXT
	
#ifdef L_vq_gdos
	
	XDEF	vq_gdos

vq_gdos:

#ifdef __MSHORT__
   move.w	#-2,d0
	trap	#2
	cmp.w	#-2,d0
	sne	d0
	ext.w	d0
#else
   move.w	#-2,d0
	trap	#2
	cmp.w	#-2,d0
	sne	d0
	ext.w	d0
	ext.l	d0
	rts 
#endif

#endif /* L_vq_gdos */

#ifdef L_vq_vgdos

/*
 * return type of GDOS
 * -2L	 no GDOS installed
 *  0x5F46534DL	 '_FSM' gdos
 *  0x5F464E54L	 '_FNT' gdos
 */
 
 	XDEF	vq_vgdos
 	
vq_vgdos:
 	moveq	#-2,d0
 	trap    #2
	rts
#endif /* L_vq_vgdos */
